import greenfoot.Actor;
import greenfoot.GreenfootImage;
// import java.awt.image.BufferedImage;
// import java.awt.Graphics2D;
import java.awt.Color;

/**
 * DrawUtil
 * 
 * @author Taylor Born
 * @version October 2013
 */
public class DrawUtil 
{
    private int playerX;
    private int playerY;
    
    private double eyeX;
    private double eyeY;
    private double centerDirectionRad;
    private double perpendicularDirectionRad;
    private double perpendicularDirectionRad_x;
    private double perpendicularDirectionRad_y;
            
    private double screenCenterDirX;
    private double screenCenterDirY;
    
    private ScreenHalf leftHalf = new ScreenHalf(0, Screen.WIDTH / 2);
    private ScreenHalf rightHalf = new ScreenHalf(Screen.WIDTH / 2, Screen.WIDTH);
    
    private long lastUpdateBackground;
    private int[] fpsLog = new int[10];
    private int firstFpsIndex;
    private int totalFPS;
    
    public DrawUtil()
    {
        Screen.main.addObject(leftHalf, Screen.WIDTH / 4, Screen.HEIGHT / 2);
        Screen.main.addObject(rightHalf, Screen.WIDTH * 3 / 4, Screen.HEIGHT / 2);
    }
    public boolean isDrawing()
    {
        return leftHalf.isDrawing() || rightHalf.isDrawing();
    }
    public void update()
    {
        playerX = (int)Player.main.getX();
        playerY = (int)Player.main.getY();
        
        eyeX = (int)Player.main.getX() + 0.5;
        eyeY = (int)Player.main.getY() + 0.5;
        centerDirectionRad = Math.toRadians(Player.main.getRotation());
        perpendicularDirectionRad = centerDirectionRad + Math.PI / 2;
        perpendicularDirectionRad_x = Math.cos(perpendicularDirectionRad);
        perpendicularDirectionRad_y = Math.sin(perpendicularDirectionRad);
            
        screenCenterDirX = 0.5 * Math.cos(centerDirectionRad);
        screenCenterDirY = 0.5 * Math.sin(centerDirectionRad);
        
        leftHalf.update();
        rightHalf.update();
        
        long currentTime = System.currentTimeMillis();
        totalFPS -= fpsLog[firstFpsIndex];
        fpsLog[firstFpsIndex] = 1000 / (int)(currentTime - lastUpdateBackground);
        totalFPS += fpsLog[firstFpsIndex];
        leftHalf.drawString("FPS: " + (totalFPS / fpsLog.length), 1);
        leftHalf.drawString("Sprite: " + (Raycaster.isShowMovingObjects() ? 'Y' : 'N'), 2);
        firstFpsIndex = (firstFpsIndex + 1) % fpsLog.length;
        lastUpdateBackground = currentTime;
    }
    
    private class ScreenHalf extends Actor implements Runnable
    {
        private GreenfootImage[] gfBuffers = new GreenfootImage[]{ new GreenfootImage(Screen.WIDTH / 2, Screen.HEIGHT) , new GreenfootImage(Screen.WIDTH / 2, Screen.HEIGHT) };
        private FastImage[] fastImageBuffers = new FastImage[]{ new FastImage(gfBuffers[0]) , new FastImage(gfBuffers[1]) };
        private int buffer;
        private volatile boolean drawing;
        private Raycaster rc = new Raycaster();
        private int start, end;
        
        public ScreenHalf(int s, int e)
        {
            start = s;
            end = e;
            setImage(gfBuffers[1]);
            (new Thread(this)).start();
        }
        public void update()
        {
            setImage(gfBuffers[buffer]);
            buffer = 1 - buffer;
            drawing = true;
        }
        public void drawString(String s, int line)
        {
            getImage().setColor(Color.WHITE);
            getImage().drawString(s, 1, 13 * line);
        }
        public boolean isDrawing()
        {
            return drawing;
        }
        public void run()
        {
            while (true)
            {
                while (!drawing)
                {}
                Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
                
//                 Graphics2D graphics = gfBuffers[buffer].getAwtImage().createGraphics();
                for (int c = start; c < end; c++)
                {
                    double across = c / (double)Screen.WIDTH - 0.5;
                    double screenDirX = screenCenterDirX + perpendicularDirectionRad_x * across;
                    double screenDirY = screenCenterDirY + perpendicularDirectionRad_y * across;
                    double planeDist = Math.sqrt(0.25 + across * across);//0.5 * 0.5
                    
                    rc.followLine(playerX, playerY, screenDirX, screenDirY, planeDist);
                    
                    for (WallIntersect intersect : rc)
                        intersect.draw(fastImageBuffers[buffer], c, planeDist);
                }
//                 graphics.dispose();
                drawing = false;
                Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
            }
        }
    }
}