import greenfoot.World;
import greenfoot.GreenfootImage;
import greenfoot.Greenfoot;
import greenfoot.MouseInfo;
import java.util.ArrayList;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Point;

/**
 * Editor
 * 
 * @author Taylor Born
 * @version October 2013
 */
public class Editor extends World
{
    public static Editor main;
    private static final int WIDTH = 600;
    private static final int HEIGHT = 400;
    static
    {
        new Editor();
    }

    private final int DIVIDE = WIDTH - 200;
    private final int CELL_SIZE = 28;
    private final int TREE_RADIUS = 3;
    private final int WINDOW_GAP = 3;
    private final Color SELECTED_COLOR = new Color(255, 0, 0, 150);
    private final Color BORDER_COLOR = new Color(200, 200, 200);
    
    private enum Mode
    {
        TREES, WALLS, ELEVATORS
    }
    private Mode mode = Mode.TREES;
    
    private GreenfootImage pic = new GreenfootImage(WIDTH, HEIGHT);
    private int scrollX;
    private int scrollY;
//     private int selectedX = -1;
//     private int selectedY = -1;
    
    private int MAX_SCROLL_Y = Map.main.getTiles().length * CELL_SIZE - HEIGHT;
    private int MAX_SCROLL_X = Map.main.getTiles()[0].length * CELL_SIZE - DIVIDE;
    
    private Container mainContainer = new Container(new Point(1, 2));
    private Container wallContainer = new Container(new Point(1, 4));
    private Container elevatorContainer = new Container(new Point(1, 3));
    
    private CheckBoxLabel NW_Wall = new CheckBoxLabel("NW", true);
    private CheckBoxLabel SE_Wall = new CheckBoxLabel("SE", true);
    
    private DropDownList<String> modeSelector = new DropDownList<String>(new ArrayList<String>(){{ add("Trees"); add("Walls"); add("Elevator"); }}, 0);
    private DropDownList<String> wallType = new DropDownList<String>(new ArrayList<String>(){{ add("Red"); add("Red_Window"); add("Blue"); add("Blue_Window"); }}, 0);
    private DropDownList<String> wallNorthOrSouth = new DropDownList<String>(new ArrayList<String>(){{ add("North"); add("West"); }}, 0);
    private DropDownList<String> elevatorDirection = new DropDownList<String>(new ArrayList<String>(){{ add("North"); add("East"); add("South"); add("West"); }}, 0);
    private ListBox<ElevatorWall> elevatorList = new ListBox<ElevatorWall>(new Point(140, 150), Map.main.getElevators());
    private CheckBoxLabel[] elevatorFloorAccess = new CheckBoxLabel[Map.MAX_FLOORS];
    private Container elevatorFloorAccessContainer = new Container(new Point(1, Map.MAX_FLOORS));

    public Editor()
    {
        super(WIDTH, HEIGHT, 1);
        main = this;
        setBackground(pic);
        draw();
        
        wallContainer.addComponent(wallType);
        wallContainer.addComponent(wallNorthOrSouth);
        wallContainer.addComponent(NW_Wall);
        wallContainer.addComponent(SE_Wall);
        
        elevatorContainer.addComponent(elevatorDirection);
        elevatorContainer.addComponent(elevatorList);
        
        for (int i = 0; i < Map.MAX_FLOORS; i++)
        {
            elevatorFloorAccess[i] = new CheckBoxLabel("Floor " + (i + 1) + " access", true);
            elevatorFloorAccessContainer.addComponent(elevatorFloorAccess[i]);
        }
        elevatorContainer.addComponent(elevatorFloorAccessContainer);
        
        mainContainer.setMinHeight(HEIGHT);
        mainContainer.addComponent(modeSelector);
        addObject(mainContainer, DIVIDE + (WIDTH - DIVIDE) / 2, HEIGHT / 2);
    }
    public void setAsWorld()
    {
        Greenfoot.setWorld(this);
        draw();
    }
    
    public void act()
    {
        boolean draw = false;
        
        if (Greenfoot.mouseClicked(this))
        {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            int column = (scrollX + mouse.getX()) / CELL_SIZE;
            int row = (scrollY + mouse.getY()) / CELL_SIZE;
            if (mouse.getX() < DIVIDE && column < Map.main.getTiles()[0].length && row < Map.main.getTiles().length)
            {
                switch (mode)
                {
                    case TREES:     
                                    Map.main.addTree(0, 0, (CELL_SIZE / 4 + scrollX + mouse.getX()) / (CELL_SIZE / 2), (CELL_SIZE / 4 + scrollY + mouse.getY()) / (CELL_SIZE / 2));
                                    break;
                    case WALLS:     
                                    Map.main.setWall(row, column, wallNorthOrSouth.getIndex() == 0, mouse.getButton() == 3 ? -1 : wallType.getIndex(), NW_Wall.isChecked(), SE_Wall.isChecked());
                                    break;
                    case ELEVATORS: 
                                    int dir = elevatorDirection.getIndex();
                                    Map.main.newElevator(column, row, (dir == 1 ? 1 : dir == 3 ? -1 : 0), (dir == 0 ? -1 : dir == 2 ? 1 : 0), Map.main.getFloorNumber() + "");
                }
                draw = true;
            }
        }
        
        if (KeysPressed.check("="))
            if (Map.main.getFloorNumber() < Map.MAX_FLOORS - 1)
            {
                Map.main.moveBetweenFloors(1);
                draw = true;
            }
        if (KeysPressed.check("-"))
            if (Map.main.getFloorNumber() > 0)
            {
                Map.main.moveBetweenFloors(-1);
                draw = true;
            }
        
        if (Greenfoot.isKeyDown("w"))
            if (scrollY > 0)
            {
                scrollY -= 3;
                if (scrollY < 0)
                    scrollY = 0;
                draw = true;
            }
        if (Greenfoot.isKeyDown("s"))
            if (scrollY < MAX_SCROLL_Y)
            {
                scrollY += 3;
                if (scrollY > MAX_SCROLL_Y)
                    scrollY = MAX_SCROLL_Y;
                draw = true;
            }
        if (Greenfoot.isKeyDown("a"))
            if (scrollX > 0)
            {
                scrollX -= 3;
                if (scrollX < 0)
                    scrollX = 0;
                draw = true;
            }
        if (Greenfoot.isKeyDown("d"))
            if (scrollX < MAX_SCROLL_X)
            {
                scrollX += 3;
                if (scrollX > MAX_SCROLL_X)
                    scrollX = MAX_SCROLL_X;
                draw = true;
            }
        
        
        if (KeysPressed.check("q"))
            modeSelector.setIndex((modeSelector.getIndex() + 1) % modeSelector.getList().size());
        if (modeSelector.hasChanged())
        {
            cleanUpLastMode();
            switch (modeSelector.getIndex())
            {
                case 0: mode = Mode.TREES; break;
                case 1: mode = Mode.WALLS;
                        mainContainer.addComponent(wallContainer, new Point(0, 1));
                        break;
                case 2: mode = Mode.ELEVATORS;
                        mainContainer.addComponent(elevatorContainer, new Point(0, 1));
            }
        }
        
        switch (modeSelector.getIndex())
        {
            case 0: break;
            case 1: for (int i = 0; i < wallType.getList().size(); i++)
                    {
                        if (Greenfoot.isKeyDown("" + (i + 1)))
                            wallType.setIndex(i);
                    }
                    if (KeysPressed.check("e"))
                        wallNorthOrSouth.setIndex(1 - wallNorthOrSouth.getIndex());
                    if (KeysPressed.check("r"))
                        NW_Wall.setChecked(!NW_Wall.isChecked());
                    if (KeysPressed.check("f"))
                        SE_Wall.setChecked(!SE_Wall.isChecked());
                    break;
            case 2: 
                    // Only show attribute fields if have selection
                    elevatorFloorAccessContainer.hide(!elevatorList.hasSelection());
                    // Selected ElevatorWall changed
                    if (elevatorList.hasChanged())
                    {
                        ElevatorWall elevator = elevatorList.getOneSelection();
                        if (elevator != null)
                            // Update attributes to match those of ElevatorWall
                            for (int i = 0; i < Map.MAX_FLOORS; i++)
                            {
                                elevatorFloorAccess[i].setChecked(elevator.hasAccessToFloor(i));
                                elevatorFloorAccess[i].hasChanged(); // Burn listener
                            }
                    }
                    // Look for attribute changes by user
                    if (elevatorList.hasSelection())
                        for (int i = 0; i < Map.MAX_FLOORS; i++)
                            if (elevatorFloorAccess[i].hasChanged())
                                // Check if legal to modify attribute, at same time making changes within Map if goes through
                                if (!Map.main.grantElevatorFloorAccess(elevatorList.getOneSelection(), i, elevatorFloorAccess[i].isChecked()))
                                {
                                    // Not legal, so change CheckBoxLabel status back
                                    elevatorFloorAccess[i].setChecked(!elevatorFloorAccess[i].isChecked());
                                    elevatorFloorAccess[i].hasChanged(); // Burn listener
                                }
                                else
                                {
                                    // Elevator has no floor access, remove it
                                    if (elevatorList.getOneSelection().getMaxFloorAccess() == -1)
                                        elevatorList.removeSelected();
                                    draw = true;
                                }
        }
        
        if (draw)
            draw();
        
        KeysPressed.update();
        if (KeysPressed.check("escape"))
            Screen.main.setAsWorld();
    }
    private void cleanUpLastMode()
    {
        switch (mode)
        {
            case TREES:     break;
            case WALLS:     wallContainer.removeFromWorld();
                            mainContainer.addComponent(null, new Point(0, 1));
                            break;
            case ELEVATORS: elevatorContainer.removeFromWorld();
                            mainContainer.addComponent(null, new Point(0, 1));
        }
    }
    private void draw()
    {
        pic.clear();
        pic.setColor(Color.WHITE);
        pic.fill();
        pic.setColor(Color.BLACK);
        pic.drawLine(DIVIDE, 0, DIVIDE, HEIGHT);
        
        Map map = Map.main;
        
        Tile[][] tiles = map.getTiles();
        
        Graphics2D graphics = (Graphics2D)pic.getAwtImage().getGraphics();
        graphics.setClip(0, 0, Math.min(tiles[0].length * CELL_SIZE + 1, DIVIDE), Math.min(tiles.length * CELL_SIZE + 1, HEIGHT));
        
        int beginDrawYAt = -scrollY % CELL_SIZE;
        int beginDrawXAt = -scrollX % CELL_SIZE;
        
        // Draw cell borders
        graphics.setColor(BORDER_COLOR);
        // Horizontal borders
        for (int y = beginDrawYAt; y < HEIGHT; y += CELL_SIZE)
            graphics.drawLine(0, y, DIVIDE, y);
        // Vertical borders
        for (int x = beginDrawXAt; x < DIVIDE; x += CELL_SIZE)
            graphics.drawLine(x, 0, x, HEIGHT);
        
        int firstRowToDraw = scrollY / CELL_SIZE;
        int firstColToDraw = scrollX / CELL_SIZE;
        
        for (int r = firstRowToDraw; r < tiles.length; r++)
        {
            // Calculate screen y position for cells in row
            int cellDrawY = beginDrawYAt + (r - firstRowToDraw) * CELL_SIZE;
            // If beyond view, row does not need to be drawn
            if (cellDrawY > HEIGHT)
                break;
            
            for (int c = firstColToDraw; c < tiles[0].length; c++)
            {
                // Calculate screen x position for cell
                int cellDrawX = beginDrawXAt + (c - firstColToDraw) * CELL_SIZE;
                // If beyond view, cell does not need to be drawn
                if (cellDrawX > DIVIDE)
                    break;
                
                if (Map.main.elevatorAt(c, r, Map.main.getFloorNumber()))// || (r == selectedY && c == selectedX))// Draw highlight background for cell if selected
                {
                    graphics.setColor(SELECTED_COLOR);
                    graphics.fillRect(cellDrawX, cellDrawY, CELL_SIZE, CELL_SIZE);
                }
                
                // Draw walls of cell
                graphics.setColor(Color.BLACK);
                
                // North wall
                
                boolean[] attributes = tiles[r][c].hasNorthWallAndIsWindow();
                
                if (attributes[0])
                    if (attributes[1])
                    {
                        graphics.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
                        graphics.fillRect(cellDrawX + CELL_SIZE / 2 + WINDOW_GAP, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
                    }
                    else
                        graphics.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE, 2);
                
//                 Wall wall = tiles[r][c].getNorthWall(true);
//                 if (wall != null)
//                     if (wall.isWindow())
//                     {
//                         graphics.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
//                         graphics.fillRect(cellDrawX + CELL_SIZE / 2 + WINDOW_GAP, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
//                     }
//                     else
//                         graphics.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE, 2);
                // West wall
                
                attributes = tiles[r][c].hasWestWallAndIsWindow();
                
                if (attributes[0])
                    if (attributes[1])
                    {
                        graphics.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE / 2 - WINDOW_GAP);
                        graphics.fillRect(cellDrawX - 1, cellDrawY + CELL_SIZE / 2 + WINDOW_GAP, 2, CELL_SIZE / 2 - WINDOW_GAP);
                    }
                    else
                        graphics.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE);
                
//                 wall = tiles[r][c].getWestWall(true);
//                 if (wall != null)
//                     if (wall.isWindow())
//                     {
//                         graphics.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE / 2 - WINDOW_GAP);
//                         graphics.fillRect(cellDrawX - 1, cellDrawY + CELL_SIZE / 2 + WINDOW_GAP, 2, CELL_SIZE / 2 - WINDOW_GAP);
//                     }
//                     else
//                         graphics.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE);
                
                // Draw trees of cell
                graphics.setColor(Color.GREEN);
                boolean[][] trees = map.getCellTrees(c, r);
                for (int i = 0; i < trees.length; i++)
                    for (int k = 0; k < trees[0].length; k++)
                        if (trees[i][k])
                            graphics.fillOval(cellDrawX + CELL_SIZE / 2 * k - TREE_RADIUS, cellDrawY + CELL_SIZE / 2 * i - TREE_RADIUS, TREE_RADIUS * 2, TREE_RADIUS * 2);
                
            }
        }
        
        graphics.dispose();
    }
    private boolean isElevatorHere(int r, int c, Wall w)
    {
        return w != null && w instanceof ElevatorWall && ((ElevatorWall)w).isLocatedAt(c, r);
    }
}