import greenfoot.Greenfoot;
import greenfoot.GreenfootImage;
import greenfoot.GreenfootSound;
import java.awt.image.BufferedImage;
import java.awt.Color;

/**
 * ElevatorWall
 * 
 * @author Taylor Born
 * @version October 2013
 */
public class ElevatorWall extends Wall
{
    public static DynamicWall wallpaper1;
    public static DynamicWall wallpaper2;//was slice
    
    private final int DOOR_CLOSED = 25;
    private final int DOOR_OPEN = 0;
    private final double DOOR_RATE = 0.5;
    private static int[][][] doorColors = new int[6][100][4];
    private final int TRAVEL_TIME = 900;
    private static final int NULL_FLOOR = -1;
    
    private static final int[][] CODED_BTN_RGB = getColorArray(Color.MAGENTA); // Color used to indicate buttons in image
    private final int[][] DEFAULT_BTN_RGB = getColorArray(new Color(133, 113, 83)); // Color for default state of a button. Dark tan
    private final int[][] SELECTED_BTN_RGB = getColorArray(Color.WHITE); // Color for floor button of the floor the elevator is moving to
    private final int[][] CURRENT_FLR_BTN_RGB = getColorArray(Color.BLACK); // Color for floor button of the floor currently at
    private final int[][] HOVER_BTN_RGB = getColorArray(Color.YELLOW); // Color for button when mouse if hovering over it
    private final int[][] CRACK_RGB = getColorArray(Color.BLACK); // Color for crack in floor
//     private final int NULL_RGB = -1; // For when no color is to be drawn
    
    private static GreenfootSound music = new GreenfootSound("music.wav");
    private static GreenfootSound[] doorSound = new GreenfootSound[]{ new GreenfootSound("DoorOpening.wav"), new GreenfootSound("DoorOpening.wav") };
    private static boolean firstDoorSound = true;
    
    private final int PANEL_LEFT = 81;
    private final int PANEL_TOP = 30;
    private static final int PANEL_WIDTH = 13;
    private static final int PANEL_HEIGHT = 27;
    private static int[][][][] panelColors = new int[PANEL_WIDTH][PANEL_HEIGHT][100][4];
    private final int OUTSIDE_BTNS_DIVIDE = 42;
    
    private final int DOOR_HEADER = 14;
    private static final int FLOOR_NUMBER_DISPLAY_HEIGHT = 12;
    private final int FLOOR_NUMBER_DISPLAY_TOP = (DOOR_HEADER - FLOOR_NUMBER_DISPLAY_HEIGHT) / 2;
    private static final int FLOOR_NUMBER_DISPLAY_WIDTH = 10;
    private final int FLOOR_NUMBER_DISPLAY_LEFT = (SIZE - FLOOR_NUMBER_DISPLAY_WIDTH) / 2;
    private static int[][][][][] floorDisplayColors = new int[Map.MAX_FLOORS][FLOOR_NUMBER_DISPLAY_WIDTH][FLOOR_NUMBER_DISPLAY_HEIGHT][100][4];
    
    // Outside panel indicating elevator's current floor
    private final int FLOOR_INDICATOR_PANEL_WIDTH = Map.MAX_FLOORS * 2 + (Map.MAX_FLOORS - 1) * 2 + 2;
    private final int FLOOR_INDICATOR_PANEL_HEIGHT = 2 + 2;
    private final int FLOOR_INDICATOR_PANEL_LEFT = (SIZE - FLOOR_INDICATOR_PANEL_WIDTH) / 2;
    private final int FLOOR_INDICATOR_PANEL_TOP = DOOR_HEADER / 2 - FLOOR_INDICATOR_PANEL_HEIGHT / 2;
    private final int[][] INDICATOR_BACK_RGB = getColorArray(Color.BLACK);
    private final int[][] INDICATOR_CURR_RGB = getColorArray(Color.YELLOW);
    private final int[][] INDICATOR_PLYR_RGB = getColorArray(Color.WHITE);
    private final int[][] INDICATOR_NULL_RGB = getColorArray(Color.GRAY);
    
    private static int[][] getColorArray(Color color)
    {
        int[][] colorArray = new int[100][4];
        colorArray[99] = new int[]{ color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha() };
        
        for (int i = 0; i < 99; i++)
            colorArray[i] = new int[]{ (int)(colorArray[99][0] * (i / 100d)), (int)(colorArray[99][1] * (i / 100d)), (int)(colorArray[99][2] * (i / 100d)), 255 };
        
        return colorArray;//new int[]{ color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha() };
    }
    private static boolean colorsEqual(int[] a, int[] b)
    {
        return a[0] == b[0] && a[1] == b[1] && a[2] == b[2];
    }
    
    static
    {
//         music.playLoop();
        music.setVolume(0);
//         SoundUtil.submitSound(music, true);
        
        doorSound[0].setVolume(85);
        doorSound[1].setVolume(85);
        
        // Door colors
        for (int i = 0; i < doorColors.length; i++)
        {
            int g = 110 + i * 8;
            doorColors[i] = getColorArray(new Color(g, g, g));//new int[]{ g, g, g, 255 };//(new Color(g, g, g)).getRGB();
        }
        
        // Outside panel
        GreenfootImage outsidePanel = new GreenfootImage("ElevatorPanel.png");
        for (int c = 0; c < panelColors.length; c++)
            for (int r = 0; r < panelColors[0].length; r++)
                panelColors[c][r] = getColorArray(outsidePanel.getColorAt(c, r));
        
        // Floor number displays
        int[][] redRGB = getColorArray(Color.RED); // Lettering
        int[][] blackRGB = getColorArray(Color.BLACK); // Back
        for (int f = 0; f < floorDisplayColors.length; f++)
        {
            GreenfootImage floorNumPic = new GreenfootImage("Floor" + (f + 1) + ".png");
            for (int c = 0; c < floorDisplayColors[0].length; c++)
                for (int r = 0; r < floorDisplayColors[0][0].length; r++)
                    floorDisplayColors[f][c][r] = (floorNumPic.getColorAt(c, r).getRed() == 0 ? blackRGB : redRGB);
        }
    }
    
    /**
     * Update the volume of the music. If the player is inside the elevator, the volume becomes audible.
     * Otherwise, the volume ebbs off.
     * @param inElevator Whether the Player is inside any elevator.
     */
    public static void updateMusic(boolean inElevator)
    {
        int volume = music.getVolume();
        if (inElevator)
        {
            if (volume < 80)
            {
                if (volume == 0)
                    music.playLoop();
                music.setVolume(volume + 4);
            }
        }
        else if (volume > 0)
        {
            music.setVolume(volume - 1);
            if (volume == 1)
                music.stop();
        }
    }
    
    private enum MouseOverBtn
    {
        NONE, UP, DOWN
    }
    private MouseOverBtn lastBtn, newBtn;
    
    private int lastFloorBtn, newFloorBtn;
    private int goToFloor = NULL_FLOOR;
    
    private long lastFloorTime;
    private boolean preparing;
    
    private double doorState = DOOR_OPEN; // State of the door
    private boolean closing = false; // The current goal of the door
    
    private int x, y; // Location in the map
    private int entryDX, entryDY; // Offset to location of just outside the elevator
    
    private int[][][][] floorChoicePanelColors = new int[PANEL_WIDTH][PANEL_HEIGHT][100][4];
    private boolean[] floorAccess = new boolean[Map.MAX_FLOORS];
    private int minFloorAccess;
    private int maxFloorAccess;
    private int currentFloor;

    public ElevatorWall(int x, int y, int entryDX, int entryDY, String floors)
    {
        super(wallpaper1.getPic());
        
        minFloorAccess = floors.charAt(0) - '0';
        currentFloor = minFloorAccess;
        maxFloorAccess = floors.charAt(floors.length() - 1) - '0';
        for (int i = 0; i < floors.length(); i++)
            floorAccess[floors.charAt(i) - '0'] = true;
        window = true;
        
        setFloorChoicePanelColors();
        
        this.x = x;
        this.y = y;
        this.entryDX = entryDX;
        this.entryDY = entryDY;
    }
    private void setFloorChoicePanelColors()
    {
        // Floor button panel
        int[][] gray1RGB = getColorArray(new Color(75, 75, 75)); // Border
        int[][] gray2RGB = getColorArray(new Color(57, 57, 57)); // Inner
        for (int c = 0; c < floorChoicePanelColors.length; c++)
            for (int r = 0; r < floorChoicePanelColors[0].length; r++)
                if (c == 0 || c == floorChoicePanelColors.length - 1 || r == 0 || r == floorChoicePanelColors[0].length - 1)
                    floorChoicePanelColors[c][r] = gray1RGB;
                else if (button(c, r) != NULL_FLOOR)
                    floorChoicePanelColors[c][r] = CODED_BTN_RGB;
                else
                    floorChoicePanelColors[c][r] = gray2RGB;
    }
    public int getX()
    {
        return x;
    }
    public int getY()
    {
        return y;
    }
    public int getEntryDX()
    {
        return entryDX;
    }
    public int getEntryDY()
    {
        return entryDY;
    }
    public int getMinFloorAccess()
    {
        return minFloorAccess;
    }
    public int getMaxFloorAccess()
    {
        return maxFloorAccess;
    }
    
    public boolean hasAccessToFloor(int floor)
    {
        return floorAccess[floor];
    }
    public boolean hasAccessToThisFloor()
    {
        return hasAccessToFloor(Map.main.getFloorNumber());
    }
    public void setAccessToFloor(int floor, boolean access)
    {
        floorAccess[floor] = access;
        if (access)
        {
            if (floor < minFloorAccess)
                minFloorAccess = floor;
            if (floor > maxFloorAccess)
                maxFloorAccess = floor;
        }
        else
        {
            if (floor == minFloorAccess)
                for (int i = minFloorAccess + 1; i <= Map.MAX_FLOORS; i++)
                    if (i == Map.MAX_FLOORS || floorAccess[i])
                    {
                        minFloorAccess = i;
                        break;
                    }
            if (floor == maxFloorAccess)
                for (int i = maxFloorAccess - 1; i >= -1; i--)
                    if (i == -1 || floorAccess[i])
                    {
                        maxFloorAccess = i;
                        break;
                    }
        }
        setFloorChoicePanelColors();
    }
    
    public boolean runsThroughFloor(int floor)
    {
        return floor >= minFloorAccess && floor <= maxFloorAccess;
    }
    public boolean runsThroughThisFloor()
    {
        return runsThroughFloor(Map.main.getFloorNumber());
    }
    public boolean opensTo(int x, int y, int floor)
    {
        return runsThroughFloor(floor) && this.x + entryDX == x && this.y + entryDY == y;
    }
    
    public boolean isLocatedAt(int x, int y)
    {
        return this.x == x && this.y == y;
    }
    
    /**
     * Determine which floor button the given location is over relative to inside the panel.
     * @param c The column within the panel.
     * @param r The row within the panel.
     * @return The floor number indexed from 0, of the button the given location is over relative to inside the panel. -1 if not over a button.
     */
    private int button(int c, int r)
    {
        // Check each button
        for (int i = 0; i < Map.MAX_FLOORS; i++)
            if (floorAccess[i])
            {
                // Location of top most pixel of button
                int x = (i % 2 == 0 ? 3 : PANEL_WIDTH - 1 - 3);
                int y = PANEL_HEIGHT - 8 - (i / 2) * 8;
                
                // Check if given location is at the button's top pixel, its middle, or its bottom pixel
                if ((c == x && r == y) ||
                    (c >= x - 1 && c <= x + 1 && r > y && r <= y + 3) ||
                    (c == x && r == y + 4))
                    return i;
            }
        return NULL_FLOOR;
    }
    
    /**
     * Allow elevator to operate.
     */
    public void operate()
    {
        lastBtn = newBtn;
        newBtn = MouseOverBtn.NONE;
        
        lastFloorBtn = newFloorBtn;
        newFloorBtn = NULL_FLOOR;
        
        operateChangingFloors();
        
        updateDoor();
        
        checkMousePress();
    }
    
    /**
     * If elevator is moving, do appropriate operations.
     */
    private void operateChangingFloors()
    {
        if (goToFloor != NULL_FLOOR)
            if (preparing) // Door is closing
            {
                if (doorState == DOOR_CLOSED - DOOR_RATE) // Door will now be closed this time
                {
                    lastFloorTime = System.currentTimeMillis(); // Record time we left this floor
                    preparing = false;
                }
            }
            else
            {
                long currentTime = System.currentTimeMillis();
                if (currentTime - lastFloorTime > TRAVEL_TIME) // Time has elapsed, we reached next floor
                {
                    if (currentFloor == goToFloor) // Check if at destination
                    {
                        closing = false; // Open door
                        goToFloor = NULL_FLOOR;
                    }
                    else
                    {
                        if (inElevator())
                            Map.main.moveToFloorNumber(goToFloor); // Update floor
                        
                        if (goToFloor > currentFloor)
                            currentFloor++;
                        else
                            currentFloor--;
                        
                        lastFloorTime = currentTime; // Record time we left this floor
                    }
                }
            }
    }
    
    /**
     * Continue closing or opening door.
     */
    private void updateDoor()
    {
        if (closing)
        {
            if (doorState < DOOR_CLOSED)
            {
                if (doorState == DOOR_OPEN)
                    playDoorSound();
                doorState += DOOR_RATE;
            }
        }
        else if (doorState > DOOR_OPEN)
        {
            if (doorState == DOOR_CLOSED)
                playDoorSound();
            doorState -= DOOR_RATE;
        }
    }
    
    private void playDoorSound()
    {
        if (currentFloor != Map.main.getFloorNumber())
            return;
        if (firstDoorSound)
            doorSound[0].play();
        else
            doorSound[1].play();
        firstDoorSound = !firstDoorSound;
    }
    
    /**
     * Check whether the mouse was pressed and if a button was hovered, act accordingly.
     */
    private void checkMousePress()
    {
        if (Greenfoot.mousePressed(null))
            if (lastFloorBtn != NULL_FLOOR)
            {
                if (doorState == DOOR_OPEN)
                {
                    closing = true;
                    goToFloor = lastFloorBtn;
                    preparing = true;
                }
            }
            else if (lastBtn != MouseOverBtn.NONE)
                if (currentFloor != Map.main.getFloorNumber() && goToFloor == NULL_FLOOR && doorState == DOOR_OPEN)
                {
                    closing = true;
                    goToFloor = Map.main.getFloorNumber();
                    preparing = true;
                }
    }
    
    public boolean isWindow(int column)
    {
        return currentFloor == Map.main.getFloorNumber() && column > 24 + (int)doorState && column < SIZE - 25 - (int)doorState;
    }
    private boolean isSide(int column)
    {
        return column < 25 || column > SIZE - 26;
    }
    
    /**
     * Determine whether this wall cannot be moved through.
     * @return If the door is not fully open.
     */
    public boolean isBarrier()
    {
        return currentFloor != Map.main.getFloorNumber() || doorState != DOOR_OPEN;
    }
    
    /**
     * Determine whether the player is inside the elevator.
     * @return Whether the player's location is within the map tile of the elevator's location.
     */
    public boolean inElevator()
    {
        return runsThroughThisFloor() && (int)Player.main.getX() / Tile.SIZE == x && (int)Player.main.getY() / Tile.SIZE == y;
    }
    
    private DynamicWall otherSide;
    public void draw(FastImage fastImage, int columnInImage, int c, int y, int height, int fade, DynamicWall otherSide)
    {
        this.otherSide = otherSide;
        draw(fastImage, columnInImage, c, y, height, fade);
    }
    
    /**
     * Draw "slice" of the elevator.
     * @param bufferedImage Image to draw to.
     * @param columnInImage The vertical "slice" of elevator's image to draw.
     * @param c Column in World.
     * @param y 
     * @param height The span the "slice" is above or below the "horizon".
     */
    public void draw(FastImage fastImage, int columnInImage, int c, int y, int height, int fade)//BufferedImage bufferedImage
    {
        double doorState = (currentFloor == Map.main.getFloorNumber() ? this.doorState : DOOR_CLOSED);
        
        if (entryDX == -1 || entryDY == 1)
            columnInImage = SIZE - 1 - columnInImage;
        
        int columnInViewHalf = c % (Screen.WIDTH / 2);
        
        boolean inElevator = inElevator();
        
        int rowInSpan = 0;
        // Start at top of view if starting point is above top of view
        if (Screen.HEIGHT / 2 - height < 0)
            rowInSpan = -(Screen.HEIGHT / 2 - height);
        
        for (; rowInSpan < height * 2; rowInSpan++)
        {
            int rowInView = Screen.HEIGHT / 2 - height + rowInSpan;
            
            // If below bottom of view, are done
            if (rowInView >= Screen.HEIGHT)
                break;
            
            int rowInImage = (int)(rowInSpan / (double)(height * 2) * Wall.SIZE);
            
            // Color for pixel
            int[][] rgb = null;
            boolean hasColor = true;
            
            // To left or right of door
            if (isSide(columnInImage))
            {
                // Within bounds of panel
                if (columnInImage >= PANEL_LEFT &&
                    columnInImage < PANEL_LEFT + PANEL_WIDTH &&
                    rowInImage >= PANEL_TOP &&
                    rowInImage < PANEL_TOP + PANEL_HEIGHT)
                {
                    // Choose from which panel
                    if (inElevator)
                        rgb = floorChoicePanelColors[columnInImage - PANEL_LEFT][rowInImage - PANEL_TOP];
                    else
                        rgb = panelColors[columnInImage - PANEL_LEFT][rowInImage - PANEL_TOP];
                }
                else
                    // Whatever color from wall
                    if (inElevator)
                        rgb = wallpaper1.getColor(columnInImage, rowInImage);
                    else
                        rgb = otherSide.getColor(columnInImage, rowInImage);// colors[columnInImage][rowInImage];
                    
            }
            // Above door
            else if (rowInImage < DOOR_HEADER)
            {
                if (inElevator)
                {
                    if ( rowInImage >= FLOOR_NUMBER_DISPLAY_TOP &&
                         rowInImage < FLOOR_NUMBER_DISPLAY_TOP + FLOOR_NUMBER_DISPLAY_HEIGHT &&
                         columnInImage >= FLOOR_NUMBER_DISPLAY_LEFT &&
                         columnInImage < FLOOR_NUMBER_DISPLAY_LEFT + FLOOR_NUMBER_DISPLAY_WIDTH)
                        rgb = floorDisplayColors[Map.main.getFloorNumber()][columnInImage - FLOOR_NUMBER_DISPLAY_LEFT][rowInImage - FLOOR_NUMBER_DISPLAY_TOP];
                    else
                        rgb = wallpaper1.getColor(columnInImage, rowInImage);
                }
                else if (rowInImage >= FLOOR_INDICATOR_PANEL_TOP &&
                         rowInImage < FLOOR_INDICATOR_PANEL_TOP + FLOOR_INDICATOR_PANEL_HEIGHT &&
                         columnInImage >= FLOOR_INDICATOR_PANEL_LEFT &&
                         columnInImage < FLOOR_INDICATOR_PANEL_LEFT + FLOOR_INDICATOR_PANEL_WIDTH)
                {
                    int rowInPanel = rowInImage - FLOOR_INDICATOR_PANEL_TOP;
                    int columnInPanel = columnInImage - FLOOR_INDICATOR_PANEL_LEFT;
                    if (rowInPanel == 0 || rowInPanel == FLOOR_INDICATOR_PANEL_HEIGHT - 1 || columnInPanel == 0 || columnInPanel == FLOOR_INDICATOR_PANEL_WIDTH - 1)
                        rgb = INDICATOR_BACK_RGB;
                    else
                    {
                        columnInPanel--;
                        int floor = columnInPanel / 4;
                        if (columnInPanel % 4 < 2)
                        {
                            if (Map.MAX_FLOORS - 1 - floor == currentFloor) // Note outside elevator is reverse
                                rgb = INDICATOR_CURR_RGB;
                            else if (Map.MAX_FLOORS - 1 - floor == Map.main.getFloorNumber())
                                rgb = INDICATOR_PLYR_RGB;
                            else
                                rgb = INDICATOR_NULL_RGB;
                        }
                        else
                            rgb = INDICATOR_BACK_RGB;
                    }
                }
                else
                    rgb = otherSide.getColor(columnInImage, rowInImage);
            }
            // Door opening
            else if (isWindow(columnInImage))
            {
                // Crack in floor
                if (rowInImage == SIZE - 1)
                    rgb = CRACK_RGB;
                else
                    hasColor = false;
            }
            // The door
            else
            {
                // Inner edge of door
                if (columnInImage == SIZE / 4 - 1 + (int)doorState || columnInImage == SIZE * 3 / 4 - (int)doorState)
                    rgb = doorColors[doorColors.length - 1];
                // Left or right portion of door
                else if (columnInImage < SIZE / 2)
                    rgb = doorColors[(int)((columnInImage - (int)doorState) / (double)DOOR_CLOSED * (doorColors.length - 1))];
                else
                    rgb = doorColors[(int)(((SIZE - columnInImage - 1 - (int)doorState) / (double)DOOR_CLOSED) * (doorColors.length - 1))];
            }
            
            if (hasColor)
            {
                boolean mouseInColumn = Screen.main.getLastMouse().getX() == c;
                
                // Coded
                if (colorsEqual(rgb[99], CODED_BTN_RGB[99]))
                {
                    if (inElevator)
                    {
                        // Button that a part of
                        int btn = button(columnInImage - PANEL_LEFT, rowInImage - PANEL_TOP);
                        
                        // Elevator in motion
                        if (goToFloor != NULL_FLOOR)
                        {
                            // Is button for destination
                            if (btn == goToFloor)
                                rgb = SELECTED_BTN_RGB;
                        }
                        // Elevator not in motion
                        else
                        {
                            // Part of button that mouse was last known to be over
                            if (btn == lastFloorBtn)
                                rgb = HOVER_BTN_RGB;
                            // Part of button for current floor
                            else if (btn == Map.main.getFloorNumber())
                                rgb = CURRENT_FLR_BTN_RGB;
                            
                            // Not part of button for current floor
                            if (btn != Map.main.getFloorNumber())
                                // Mouse over this pixel
                                if (mouseInColumn && Screen.main.getLastMouse().getY() == rowInView)
                                    // Remember button mouse is over
                                    newFloorBtn = btn;
                        }
                    }
                    // Outside elevator
                    else
                    {
                        // Part of button that mouse was last known to be over
                        if ((lastBtn == MouseOverBtn.UP && rowInImage < OUTSIDE_BTNS_DIVIDE) || (lastBtn == MouseOverBtn.DOWN && rowInImage > OUTSIDE_BTNS_DIVIDE))
                            rgb = HOVER_BTN_RGB;
                        
                        // Mouse is in this column of screen
                        if (mouseInColumn)
                            // Mouse is in this row of screen. So is over this pixel
                            if (Screen.main.getLastMouse().getY() == rowInView)
                                // Remember button mouse is over
                                if (rowInImage < 42)
                                    newBtn = MouseOverBtn.UP;
                                else
                                    newBtn = MouseOverBtn.DOWN;
                    }
                    
                    // Have not replaced coded color yet
                    if (colorsEqual(rgb[99], CODED_BTN_RGB[99]))
                        rgb = DEFAULT_BTN_RGB;
                }
                
                // Set pixel to color
                fastImage.setPixel(columnInViewHalf, rowInView, rgb[fade]);
//                 bufferedImage.setRGB(columnInViewHalf, rowInView, rgb);
            }
        }
    }
    
    
    public String save()
    {
        String s = x + "," + y + ":" + entryDX + "," + entryDY + ":";
        for (int i = 0; i < floorAccess.length; i++)
            if (floorAccess[i])
                s += i;
        return s;
    }
    public String toString()
    {
        String s = "(" + x + ", " + y + ") (";
        for (int i = 0; i < floorAccess.length; i++)
            if (floorAccess[i])
                s += (i + 1);
        return s + ") (" + (entryDX == -1 ? "W" : entryDX == 1 ? "E" : entryDY == -1 ? "N" : "S") + ")";
    }
}