import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import greenfoot.GreenfootImage;
import java.util.ArrayList;
import java.awt.Point;

/**
 * Map
 * 
 * @author Taylor Born
 * @version September - October 2013
 */
public class Map 
{
    public static Map main;
    public static final int MAX_FLOORS = 6;
    public static FixtureWall treeMid, treeLeft, treeRight, treeBoth;
    public static void clear()
    {
        main = null;
        setTreeWalls(null, null, null, null);
    }
    public static void setTreeWalls(FixtureWall mid, FixtureWall left, FixtureWall right, FixtureWall both)
    {
        treeMid = mid;
        treeLeft = left;
        treeRight = right;
        treeBoth = both;
    }
    public static void save()
    {
        main.saveTiles();
    }

    private Tile[][][] tiles;
    private boolean[][][] trees;
    private Wall[] walls;// = { new Wall("Wall.png"), new Wall("Window.png") };
    private int floor = 0;
    private ArrayList<ElevatorWall> elevators = new ArrayList<ElevatorWall>();
    
    public int getFloorNumber()
    {
        return floor;
    }
    public void setFloorNumber(int floor)
    {
        this.floor = floor;
    }
    public void moveToFloorNumber(int floor)
    {
        this.floor += (floor > this.floor ? 1 : -1);
    }
    public void moveBetweenFloors(int numberOfFloors)
    {
        floor += numberOfFloors;
    }
    public int getWallIndex(Wall wall)
    {
        for (int i = 0; i < walls.length; i++)
            if (walls[i] == wall)
                return i;
        return -1;
    }

    public Map(Wall[] walls)
    {
        main = this;
        this.walls = walls;
        load();
    }
    
    public void operateElevators()
    {
        boolean inElevator = false;
        for (ElevatorWall elevator : elevators)
        {
            elevator.operate();
            if (!inElevator && elevator.inElevator())
                inElevator = true;
        }
        ElevatorWall.updateMusic(inElevator);
    }
    public ArrayList<ElevatorWall> getElevators()
    {
        return elevators;
    }
//     public void setElevator(int r, int c, int dir)
//     {
//         tiles[floor][r][c].setNorthWall(dir == 0 ? walls[2] : walls[0]);
//         tiles[floor][r][c].setWestWall(dir == 3 ? walls[2] : walls[0]);
//         if (r < tiles[floor].length - 1)
//             tiles[floor][r + 1][c].setNorthWall(dir == 2 ? walls[2] : walls[0]);
//         if (c < tiles[floor][r].length - 1)
//             tiles[floor][r][c + 1].setWestWall(dir == 1 ? walls[2] : walls[0]);
//     }
    
    
    public Tile[][] getTiles()
    {
        return tiles[floor];
    }
    public boolean outOfBounds(int x, int y)
    {
        return x < 0 || y < 0 || x > tiles[floor][0].length - 1 || y > tiles[floor].length - 1;
    }
    
    /**
     * 
     * Prevents modifying interior walls of elevators. Also won't allow nulling these walls including exterior.
     */
    public void setWall(int r, int c, boolean horizontal, int type, boolean nw, boolean se)
    {
        Wall w = (type == -1 ? null : walls[type]);
        boolean elevatorHere = elevatorAt(c, r, floor);
        if (w == null && elevatorHere)
            return;
        if (horizontal)
        {
            boolean elevatorNorth = elevatorAt(c, r - 1, floor);
            if (w == null && elevatorNorth)
                return;
            tiles[floor][r][c].setNorthWall(w, !elevatorNorth && nw, !elevatorHere && se);
        }
        else
        {
            boolean elevatorWest = elevatorAt(c - 1, r, floor);
            if (w == null && elevatorWest)
                return;
            tiles[floor][r][c].setWestWall(w, !elevatorWest && nw, !elevatorHere && se);
        }
    }
    public boolean elevatorAt(int x, int y, int floor)
    {
        for (ElevatorWall elevator : elevators)
            if (elevator.isLocatedAt(x, y) && elevator.runsThroughFloor(floor))
                return true;
        return false;
    }
    
    public void saveTiles()
    {
        System.out.println("");
        System.out.println(tiles[0].length + "," + tiles[0][0].length);
        for (int floor = 0; floor < tiles.length; floor++)
        {
            for (int r = 0; r < tiles[0].length; r++)
            {
                for (int c = 0; c < tiles[0][0].length; c++)
                    System.out.print(tiles[floor][r][c].save() + ",");
                System.out.println("");
            }
            for (int r = 0; r < trees[floor].length; r++)
                for (int c = 0; c < trees[floor][0].length; c++)
                    if (trees[floor][r][c])
                        System.out.print(r + "," + c + ";");
            System.out.println("");
        }
        for (ElevatorWall elevator : elevators)
            System.out.print(elevator.save() + ";");
        System.out.println("");
    }
    private void load()
    {
        try
        {
            BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getClassLoader().getResource("map.txt").openStream()));
            
            String str = br.readLine();
            tiles = new Tile[MAX_FLOORS][Integer.parseInt(str.substring(0, str.indexOf(",")))][Integer.parseInt(str.substring(str.indexOf(",") + 1))];
            trees = new boolean[MAX_FLOORS][tiles[0].length * 2 + 1][tiles[0][0].length * 2 + 1];
            
            for (int floor = 0; floor < tiles.length; floor++)
            {
                str = br.readLine();
                for (int r = 0; r < tiles[0].length; r++)
                {
                    for (int c = 0; c < tiles[0][0].length; c++)
                    {
                        Wall[] walls = new Wall[4];
                        for (int i = 0; i < walls.length; i++)
                        {
                            int n = str.charAt(i) - '0';
                            walls[i] = (n == 0 ? null : this.walls[n - 1]);
                        }
                        tiles[floor][r][c] = new Tile(walls);
                        
                        str = str.substring(str.indexOf(",") + 1);
                    }
                    str = br.readLine();
                }
                
                while (!str.isEmpty())
                {
                    trees[floor][Integer.parseInt(str.substring(0, str.indexOf(",")))][Integer.parseInt(str.substring(str.indexOf(",") + 1, str.indexOf(";")))] = true;
                    str = str.substring(str.indexOf(";") + 1);
                }
            }
            str = br.readLine();
            while (!str.isEmpty())
            {
                int x = Integer.parseInt(str.substring(0, str.indexOf(",")));
                int y = Integer.parseInt(str.substring(str.indexOf(",") + 1, str.indexOf(":")));
                str = str.substring(str.indexOf(":") + 1);
                int dx = Integer.parseInt(str.substring(0, str.indexOf(",")));
                int dy = Integer.parseInt(str.substring(str.indexOf(",") + 1, str.indexOf(":")));
                str = str.substring(str.indexOf(":") + 1);
                String floors = str.substring(0, str.indexOf(";"));
                newElevator(x, y, dx, dy, floors);
                str = str.substring(str.indexOf(";") + 1);
            }
            br.close();
        }
        catch (IOException e)
        {}
        
        
        if (CodeGenerator.haveUser())
        {
            final int CODE_ID = 73;
            GreenfootImage pic = new GreenfootImage("Wall.png");
            String codeString = "  ";
            for (char ch : CodeGenerator.getCode(CODE_ID).toCharArray())
                codeString += Character.toString(ch) + " ";
            codeString += " ";
            GreenfootImage code = new GreenfootImage(codeString, 12, java.awt.Color.WHITE, new java.awt.Color(0, 0, 0, 0));
            code.mirrorHorizontally(); //
            pic.drawImage(code, (pic.getWidth() - code.getWidth()) / 2, (pic.getHeight() - code.getHeight()) / 2);
            
            tiles[2][5][8].setNorthWall(new DynamicWall(pic), true, false);
        }
    }
    
    public void newElevator(int x, int y, int dx, int dy, String floors)
    {
        // Don't create new elevator if already one there, door opens into another elevator, or if elevator at adjacent Tiles with doors opening to location
        if (elevatorAt(x, y, floors.charAt(0) - '0') || elevatorAt(x + dx, y + dy, floors.charAt(0) - '0') || elevatorOpensTo(x, y, floors.charAt(0) - '0'))
            return;
        
        ElevatorWall elevator = new ElevatorWall(x, y, dx, dy, floors);
        for (int floor = floors.charAt(0) - '0'; floor <= floors.charAt(floors.length() - 1) - '0'; floor++)
            setupWallsForElevator(elevator, floor, x, y, dx, dy);
        elevators.add(elevator);
    }
    private void setupWallsForElevator(ElevatorWall elevator, int floor, int x, int y, int dx, int dy)
    {
        // Don't modify interior wallpaper of neighboring elevator
        if (!elevatorAt(x - 1, y, floor))
            tiles[floor][y][x].setWestWall(walls[0], 0, true);
        tiles[floor][y][x].setWestWall(dx == -1 ? elevator : ElevatorWall.wallpaper2, 1);
        
        // Don't modify interior wallpaper of neighboring elevator
        if (!elevatorAt(x, y - 1, floor))
            tiles[floor][y][x].setNorthWall(walls[0], 0, true);
        tiles[floor][y][x].setNorthWall(dy == -1 ? elevator : ElevatorWall.wallpaper2, 1);
        
        // Don't modify interior wallpaper of neighboring elevator
        if (!elevatorAt(x + 1, y, floor))
            tiles[floor][y][x + 1].setWestWall(walls[0], 1, true);
        tiles[floor][y][x + 1].setWestWall(dx == 1 ? elevator : ElevatorWall.wallpaper2, 0);
        
        // Don't modify interior wallpaper of neighboring elevator
        if (!elevatorAt(x, y + 1, floor))
            tiles[floor][y + 1][x].setNorthWall(walls[0], 1, true);
        tiles[floor][y + 1][x].setNorthWall(dy == 1 ? elevator : ElevatorWall.wallpaper2, 0);
    }
    private void removeElevatorWalls(int x, int y, int floor)
    {
        Wall w = tiles[floor][y][x].getWestWall(false);
        tiles[floor][y][x].setWestWall(w == ElevatorWall.wallpaper2 ? walls[0] : w, 1);
        w = tiles[floor][y][x].getNorthWall(false);
        tiles[floor][y][x].setNorthWall(w == ElevatorWall.wallpaper2 ? walls[0] : w, 1);
        w = tiles[floor][y][x + 1].getWestWall(true);
        tiles[floor][y][x + 1].setWestWall(w == ElevatorWall.wallpaper2 ? walls[0] : w, 0);
        w = tiles[floor][y + 1][x].getNorthWall(true);
        tiles[floor][y + 1][x].setNorthWall(w == ElevatorWall.wallpaper2 ? walls[0] : w, 0);
    }
    
    /**
     * 
     * @return Whether access was granted.
     */
    public boolean grantElevatorFloorAccess(ElevatorWall elevator, int floor, boolean access)
    {
        if (access)
        {
            if (elevator.runsThroughFloor(floor))
            {
                elevator.setAccessToFloor(floor, true);
                return true;
            }
            else
            {
                int min = elevator.getMinFloorAccess();
                int max = elevator.getMaxFloorAccess();
                boolean flag = false;
                int start = (floor < min ? min - 1 : max + 1);
                int step = (floor < min ? -1 : 1);
                for (int i = start; i != floor + step; i += step)
                    // Check if elevator at location, at Tile where door opens to, or at adjacent Tiles with doors opening to location
                    if (elevatorAt(elevator.getX(), elevator.getY(), i) || elevatorAt(elevator.getX() + elevator.getEntryDX(), elevator.getY() + elevator.getEntryDY(), i) || elevatorOpensTo(elevator.getX(), elevator.getY(), i))
                    {
                        flag = true;
                        break;
                    }
                if (flag)
                    return false;
                elevator.setAccessToFloor(floor, true);
                for (int i = start; i != floor + step; i += step)
                    setupWallsForElevator(elevator, i, elevator.getX(), elevator.getY(), elevator.getEntryDX(), elevator.getEntryDY());
                return true;
            }
        }
        else
        {
            int minBefore = elevator.getMinFloorAccess();
            int maxBefore = elevator.getMaxFloorAccess();
            elevator.setAccessToFloor(floor, false);
            
            // If still runs through this floor, don't need to remove any walls
            // Otherwise, remove walls on appropriate floors
            if (!elevator.runsThroughFloor(floor))
            {
                removeElevatorWalls(elevator.getX(), elevator.getY(), floor);
                
                // If floor is not both min and max access, then other floors to remove walls at
                if (floor != minBefore || floor != maxBefore)
                    if (floor == minBefore)
                    {
                        int min = elevator.getMinFloorAccess();
                        for (int i = floor; i < min; i++)
                            removeElevatorWalls(elevator.getX(), elevator.getY(), i);
                    }
                    // floor == maxBefore
                    else
                    {
                        int max = elevator.getMaxFloorAccess();
                        for (int i = floor; i > max; i--)
                            removeElevatorWalls(elevator.getX(), elevator.getY(), i);
                    }
            }
            return true;
        }
    }
    private boolean elevatorOpensTo(int x, int y, int floor)
    {
        for (ElevatorWall elevator : elevators)
            if (elevator.opensTo(x, y, floor))
                return true;
        return false;
    }
    
    public DynamicWall getTreeWall(int x, int y, boolean horizontal, int v)
    {
        if (horizontal)
        {
            v += y * 2;
            int o = x * 2;
            if (trees[floor][v][o + 1])
                return treeMid;
            if (trees[floor][v][o])
            {
                if (trees[floor][v][o + 2])
                    return treeBoth;
                return treeLeft;
            }
            if (trees[floor][v][o + 2])
                return treeRight;
        }
        else
        {
            v += x * 2;
            int o = y * 2;
            if (trees[floor][o + 1][v])
                return treeMid;
            if (trees[floor][o][v])
            {
                if (trees[floor][o + 2][v])
                    return treeBoth;
                return treeLeft;
            }
            if (trees[floor][o + 2][v])
                return treeRight;
        }
        return null;
    }
    public void addTree(int x, int y, int xOffset, int yOffset)
    {
        
        int r = y * 2 + yOffset;
        int c = x * 2 + xOffset;
        if (c == 0 || r == 0 || c == trees[floor][0].length - 1 || r == trees[floor].length - 1)
            return;
        if (trees[floor][r][c])
            trees[floor][r][c] = false;
        else
        {
            trees[floor][r][c] = true;
            if (r > 0)
                trees[floor][r - 1][c] = false;
            if (r < trees[floor].length - 1)
                trees[floor][r + 1][c] = false;
            if (c > 0)
                trees[floor][r][c - 1] = false;
            if (c < trees[floor][0].length - 1)
                trees[floor][r][c + 1] = false;
        }
    }
    public boolean isTreeAt(int x, int y, boolean midX, boolean midY)
    {
        return trees[floor][y * 2 + (midY ? 1 : 0)][x * 2 + (midX ? 1 : 0)];
    }
    public boolean[][] getCellTrees(int x, int y)
    {
        return new boolean[][]{ { trees[floor][y * 2][x * 2], trees[floor][y * 2][x * 2 + 1] }, { trees[floor][y * 2 + 1][x * 2], trees[floor][y * 2 + 1][x * 2 + 1] } };
    }
    public Point getRandomInside()
    {
        return new Point(2 + (int)(Math.random() * (getTiles()[0].length - 4)), 1 + (int)(Math.random() * (getTiles().length - 3)));
    }
}