import greenfoot.Actor;
import greenfoot.World;
import greenfoot.GreenfootImage;
import java.awt.Color;

/**
 * MiniMap
 * 
 * @author Taylor Born
 * @version October 2013
 */
public class MiniMap extends Actor
{
    private GreenfootImage pic = new GreenfootImage(160, 80);
    private GreenfootImage blank = new GreenfootImage(1, 1);
    private boolean hiding = false;

    public MiniMap()
    {
        setImage(pic);
    }
    public void act()
    {
        if (!getWorld().getObjects(LoadProgress.class).isEmpty())
            return;
        if (KeysPressed.check("m"))
        {
            if (hiding)
                setImage(pic);
            else
                setImage(blank);
            hiding = !hiding;
        }
        if (!hiding)
            draw();
    }
    private final int CELL_SIZE = 10;
    private final int TREE_RADIUS = 1;
    private final int WINDOW_GAP = 2;
    private final Color BACK_COLOR = new Color(0, 100, 0);
    
    private void draw()
    {
//         pic.clear();
        pic.setColor(BACK_COLOR);
        pic.fill();
        
        Map map = Map.main;
        Tile[][] tiles = map.getTiles();
        
        int yOffset = (int)(Player.main.getY() / Tile.SIZE * CELL_SIZE - pic.getHeight() / 2);
        int xOffset = (int)(Player.main.getX() / Tile.SIZE * CELL_SIZE - pic.getWidth() / 2);
        
        for (int r = 0; r < tiles.length; r++)
        {
            int cellDrawY = r * CELL_SIZE - yOffset;
            for (int c = 0; c < tiles[0].length; c++)
            {
                int cellDrawX = c * CELL_SIZE - xOffset;
                
                // Draw walls of cell
                pic.setColor(Color.BLACK);
                
                // North wall
                
                boolean[] attributes = tiles[r][c].hasNorthWallAndIsWindow();
                
                if (attributes[0])
                    if (attributes[1])
                    {
                        pic.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
                        pic.fillRect(cellDrawX + CELL_SIZE / 2 + WINDOW_GAP, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
                    }
                    else
                        pic.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE, 2);
                
//                 Wall wall = tiles[r][c].getNorthWall(true);
//                 if (wall != null)
//                     if (wall.isWindow())
//                     {
//                         pic.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
//                         pic.fillRect(cellDrawX + CELL_SIZE / 2 + WINDOW_GAP, cellDrawY - 1, CELL_SIZE / 2 - WINDOW_GAP, 2);
//                     }
//                     else
//                         pic.fillRect(cellDrawX, cellDrawY - 1, CELL_SIZE, 2);
                
                attributes = tiles[r][c].hasWestWallAndIsWindow();
                
                if (attributes[0])
                    if (attributes[1])
                    {
                        pic.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE / 2 - WINDOW_GAP);
                        pic.fillRect(cellDrawX - 1, cellDrawY + CELL_SIZE / 2 + WINDOW_GAP, 2, CELL_SIZE / 2 - WINDOW_GAP);
                    }
                    else
                        pic.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE);
                
                // West wall
//                 wall = tiles[r][c].getWestWall(true);
//                 if (wall != null)
//                     if (wall.isWindow())
//                     {
//                         pic.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE / 2 - WINDOW_GAP);
//                         pic.fillRect(cellDrawX - 1, cellDrawY + CELL_SIZE / 2 + WINDOW_GAP, 2, CELL_SIZE / 2 - WINDOW_GAP);
//                     }
//                     else
//                         pic.fillRect(cellDrawX - 1, cellDrawY, 2, CELL_SIZE);
                
                // Draw trees of cell
                pic.setColor(Color.GREEN);
                boolean[][] trees = map.getCellTrees(c, r);
                for (int i = 0; i < trees.length; i++)
                    for (int k = 0; k < trees[0].length; k++)
                        if (trees[i][k])
                            pic.fillOval(cellDrawX + CELL_SIZE / 2 * k - TREE_RADIUS, cellDrawY + CELL_SIZE / 2 * i - TREE_RADIUS, TREE_RADIUS * 2, TREE_RADIUS * 2);
            }
        }
        
        pic.setColor(Color.WHITE);
        for (MovingObject obj : Screen.main.getMovingObjects())
            pic.fillOval((int)(obj.getX() / Tile.SIZE * CELL_SIZE) - xOffset - 1, (int)(obj.getY() / Tile.SIZE * CELL_SIZE) - yOffset - 1, 2, 2);
        
        pic.setColor(Color.YELLOW);
        pic.drawLine(pic.getWidth() / 2, pic.getHeight() / 2 - 3, pic.getWidth() / 2, pic.getHeight() / 2 + 3);
        pic.drawLine(pic.getWidth() / 2 - 3, pic.getHeight() / 2, pic.getWidth() / 2 + 3, pic.getHeight() / 2);
        
        pic.setColor(Color.BLACK);
        pic.drawRect(0, 0, pic.getWidth() - 1, pic.getHeight() - 1);
    }
    public void addedToWorld(World world)
    {
        setLocation(world.getWidth() - pic.getWidth() / 2, pic.getHeight() / 2);
    }
}