import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * RayCaster
 * 
 * @author Taylor Born
 * @version September 2013
 */
public class Raycaster extends LineFollower implements Collection<WallIntersect>
{
    private int lastX;
    private int lastY;
    
    private int numberOfIntersects;
    private ArrayList<WallIntersect> intersects = new ArrayList<WallIntersect>();
    private IntersectIterator iterator = new IntersectIterator();
    
    private double planeDist;
    
    private ArrayList<MovingObject> objsHit = new ArrayList<MovingObject>();
    
    private boolean flag;
    
    private double dist;
    
    private boolean negativeX;
    private boolean negativeY;
    
    protected void followLine(int lineStartX, int lineStartY, double dirX, double dirY, double planeDist)
    {
        lastX = lineStartX;
        lastY = lineStartY;
        negativeX = dirX < 0;
        negativeY = dirY < 0;
        this.planeDist = planeDist;
        numberOfIntersects = 0;
        objsHit.clear();
        super.followLine(lineStartX, lineStartY, dirX, dirY);
    }
    public boolean process(int x, int y, double exactX, double exactY, boolean useX)
    {
//         int xInTile = x % Tile.SIZE;
//         int yInTile = y % Tile.SIZE;
//         if (
        
        Map map = Map.main;
        
        if (x < 0 || y < 0 || map.main.outOfBounds(x / Tile.SIZE, y / Tile.SIZE))
        {
            setLastWallIntersect();
            addIntersect(null, null, exactX, exactY, useX, WallIntersect.SKY_TYPE);
            return false;
        }
        
        dist = Player.main.getDistanceFrom(exactX, exactY) - planeDist;
        
        flag = false;
        
        if (useX)
        {
            if (lastX / Tile.SIZE == x / Tile.SIZE)
                if (lastY / Tile.SIZE != y / Tile.SIZE)
                {
                    Tile tile = map.getTiles()[y / Tile.SIZE + (y % Tile.SIZE < Tile.SIZE / 2 ? 0 : 1)][x / Tile.SIZE];
                    Wall w = tile.getNorthWall(negativeY);
                    boolean tree = false;
                    if (w == null)
                    {
                        w = map.getTreeWall(x / Tile.SIZE, y / Tile.SIZE + (y % Tile.SIZE < Tile.SIZE / 2 ? 0 : 1), true, 0);
                        tree = true;
                    }
                    if (w != null)
                    {
                        addIntersect(w, (tree ? null : tile.getNorthWall(!negativeY)), exactX, exactY, useX, tree ? WallIntersect.TREE_TYPE : WallIntersect.DEFAULT_TYPE);
//                         if (!tree && !((SliceWall)w).isWindow((int)(intersects.get(numberOfIntersects - 1).getImageOffset() * Wall.SIZE)))
//                         {
//                             flag = true;
//                             setLastWallIntersect();
//                         }
                    }
                }
                else if ((lastY % Tile.SIZE < Tile.SIZE / 2) != (y % Tile.SIZE < Tile.SIZE / 2))
                {
                    Tile tile = map.getTiles()[y / Tile.SIZE][x / Tile.SIZE];
                    Wall w = map.getTreeWall(x / Tile.SIZE, y / Tile.SIZE, true, 1);
                    if (w != null)
                        addIntersect(w, null, exactX, exactY, useX, WallIntersect.TREE_TYPE);
                }
        }
        else if (lastY / Tile.SIZE == y / Tile.SIZE)
            if (lastX / Tile.SIZE != x / Tile.SIZE)
            {
                Tile tile = map.getTiles()[y / Tile.SIZE][x / Tile.SIZE + (x % Tile.SIZE < Tile.SIZE / 2 ? 0 : 1)];
                Wall w = tile.getWestWall(negativeX);
                boolean tree = false;
                if (w == null)
                {
                    w = map.getTreeWall(x / Tile.SIZE + (x % Tile.SIZE < Tile.SIZE / 2 ? 0 : 1), y / Tile.SIZE, false, 0);
                    tree = true;
                }
                if (w != null)
                {
                    addIntersect(w, (tree ? null : tile.getWestWall(!negativeX)), exactX, exactY, useX, tree ? WallIntersect.TREE_TYPE : WallIntersect.DEFAULT_TYPE);
//                     if (!tree && !((SliceWall)w).isWindow((int)(intersects.get(numberOfIntersects - 1).getImageOffset() * Wall.SIZE)))
//                     {
//                         flag = true;
//                         setLastWallIntersect();
//                     }
                }
            }
            else if ((lastX % Tile.SIZE < Tile.SIZE / 2) != (x % Tile.SIZE < Tile.SIZE / 2))
            {
                Tile tile = map.getTiles()[y / Tile.SIZE][x / Tile.SIZE];
                Wall w = map.getTreeWall(x / Tile.SIZE, y / Tile.SIZE, false, 1);
                if (w != null)
                    addIntersect(w, null, exactX, exactY, useX, WallIntersect.TREE_TYPE);
            }
        
        if (!flag && showMovingObjects)
            for (MovingObject o : Screen.main.getMovingObjects())
                if (!objsHit.contains(o) && dist > o.distanceFromPlayer() - SQRT2)
                {
                    double leftDistance = o.distanceFromLeft(exactX, exactY);
                    double rightDistance = o.distanceFromRight(exactX, exactY);
                    if (leftDistance + rightDistance > Tile.SIZE - .5 && leftDistance + rightDistance < Tile.SIZE + .5)
                    {
                        addIntersect(o.getWall(), null, o.distanceFromPlayer() - SQRT2, 0, useX, WallIntersect.OBJECT_TYPE, rightDistance);
                        objsHit.add(o);
                        break;// ?
                    }
                }
        
        lastX = x;
        lastY = y;
        
        return !flag;
    }
    
    private static boolean showMovingObjects = false;
    public static void toggleShowingMovingObjects()
    {
        showMovingObjects = !showMovingObjects;
    }
    public static boolean isShowMovingObjects()
    {
        return showMovingObjects;
    }
    
    private final double SQRT2 = Math.sqrt(0.5);
    private void setLastWallIntersect()
    {
        for (int i = numberOfIntersects - 1; i > -1; i--)
            if (intersects.get(i).becomeLastWall())
                return;
    }
    private void addIntersect(Wall wall, Wall otherSide, double hitX, double hitY, boolean useX, int type, double hitObject)
    {
        if (intersects.size() < ++numberOfIntersects)
            intersects.add(new WallIntersect(wall, otherSide, hitX, hitY, useX, type, hitObject, dist));
        else
            intersects.get(numberOfIntersects - 1).set(wall, otherSide, hitX, hitY, useX, type, hitObject, dist);
        
        
        if (wall != null)
        {
            int checkWindow = (int)(intersects.get(numberOfIntersects - 1).getImageOffset() * Wall.SIZE);
            if ((otherSide instanceof ElevatorWall ? !otherSide.isWindow(checkWindow) : !wall.isWindow(checkWindow)))
            {
                flag = true;
                setLastWallIntersect();
            }
        }
    }
    private void addIntersect(Wall wall, Wall otherSide, double hitX, double hitY, boolean useX, int type)
    {
        addIntersect(wall, otherSide, hitX, hitY, useX, type, 0);
    }
    public Iterator<WallIntersect> iterator()
    {
        iterator.reset();
        return iterator;
    }
    public boolean isEmpty()
    {
        return numberOfIntersects == 0;
    }
    public int size()
    {
        return numberOfIntersects;
    }
    
    class IntersectIterator implements Iterator<WallIntersect>
    {
        private int index;
        
        public void reset()
        {
            index = numberOfIntersects - 1;
        }
        public boolean hasNext()
        {
            return index > -1;
        }
        public WallIntersect next()
        {
            return intersects.get(index--);
        }
        public void remove()
        {
            throw new UnsupportedOperationException();
        }
    }
    
    public boolean remove(Object o)
    {
        throw new UnsupportedOperationException();
    }
    public boolean add(WallIntersect wi)
    {
        throw new UnsupportedOperationException();
    }
    public void clear()
    {
        throw new UnsupportedOperationException();
    }
    public <T>T[] toArray(T[] a)
    {
        throw new UnsupportedOperationException();
    }
    public Object[] toArray()
    {
        throw new UnsupportedOperationException();
    }
    public boolean retainAll(Collection<?> c)
    {
        throw new UnsupportedOperationException();
    }
    public boolean removeAll(Collection<?> c)
    {
        throw new UnsupportedOperationException();
    }
    public boolean addAll(Collection<? extends WallIntersect> c)
    {
        throw new UnsupportedOperationException();
    }
    public boolean containsAll(Collection<?> c)
    {
        throw new UnsupportedOperationException();
    }
    public boolean contains(Object o)
    {
        throw new UnsupportedOperationException();
    }
}