import greenfoot.GreenfootImage;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Color;

/**
 * WallIntersect
 * 
 * @author Taylor Born
 * @version September - October 2013
 */
public class WallIntersect
{
    public static final int SKY_TYPE = 1;
    public static final int LAST_WALL_TYPE = 2;
    public static final int DEFAULT_TYPE = 3;
    public static final int TREE_TYPE = 4;
    public static final int OBJECT_TYPE = 5;
//     private static final Color CEIL_COLOR = new Color(90, 90, 90);
    private static int[][][][] skies;
    private static int[][][] ceil;
    private static int[][][] floor;
    private static int[][][] grass;
    public static void clear()
    {
        skies = null;
        grass = null;
    }
    public static void set(int[][][][] theSkies, int[][][] theCeil, int[][][] theFloor, int[][][] theGrass)
    {
        skies = theSkies;
        ceil = theCeil;
        floor = theFloor;
        grass = theGrass;
    }

    private Wall wall;
    private Wall otherSide;
    
    private double hitX;
    private double hitY;
    private boolean useX;
    private int type;
    
    private double hitObject;
    
    private int fade;

    public WallIntersect(Wall wall, Wall otherSide, double hitX, double hitY, boolean useX, int type, double hitObject, double dist)
    {
        set(wall, otherSide, hitX, hitY, useX, type, hitObject, dist);
    }
    public void set(Wall wall, Wall otherSide, double hitX, double hitY, boolean useX, int type, double hitObject, double dist)
    {
        this.wall = wall;
        this.otherSide = otherSide;
        this.hitX = hitX;
        this.hitY = hitY;
        this.useX = useX;
        this.type = type;
        this.hitObject = hitObject;
        
        fade = Math.min((int)(Math.max(1 - dist / (Tile.SIZE * 4), 0) * 100), 99);
    }
//     public double getHitX()
//     {
//         return hitX;
//     }
//     public double getHitY()
//     {
//         return hitY;
//     }
    public double getImageOffset()
    {
        if (type == OBJECT_TYPE)
            return hitObject / Tile.SIZE - Math.floor(hitObject / Tile.SIZE);
        return useX ? hitX / Tile.SIZE - Math.floor(hitX / Tile.SIZE) : hitY / Tile.SIZE - Math.floor(hitY / Tile.SIZE);
    }
    private int calculateColumnInImage()
    {
        return (int)(getImageOffset() * Wall.SIZE);
    }
//     public Wall getWall()
//     {
//         return wall;
//     }
    public boolean becomeLastWall()
    {
        if (type == DEFAULT_TYPE)
        {
            type = LAST_WALL_TYPE;
            return true;
        }
        return false;
    }
    public void draw(FastImage fastImage, int c, double planeDist)
    {
        int x = c % (Screen.WIDTH / 2);
        
        double eyeX = (int)Player.main.getX() + .5;
        double eyeY = (int)Player.main.getY() + .5;
        double dist = (type == OBJECT_TYPE ? hitX : Math.sqrt(Math.pow(hitX - eyeX, 2) + Math.pow(hitY - eyeY, 2)));
        
        int height = (int)((planeDist / dist * Tile.SIZE) * Screen.HEIGHT / 2);
//         if (height * 2 > maxH)
//             maxH = height * 2;
        int columnInImage = calculateColumnInImage();
        
        switch (type)
        {
            case SKY_TYPE:          int h = (height > skies.length ? skies.length : height);
                                    for (int y = Screen.HEIGHT / 2; y > 0; y -= h)
                                        for (int yy = 0; yy < h; yy++)
                                            fastImage.setPixel(x, y - h + yy, skies[columnInImage][h - 1][yy]);
//                                         graphics.drawImage(skies[columnInImage][h - 1], x, y - h, null);
//                                     graphics.drawImage(grass, x, Screen.HEIGHT / 2, null);
                                    for (int y = 0; y < Screen.HEIGHT / 2; y++)
                                        fastImage.setPixel(x, Screen.HEIGHT / 2 + y, grass[c][y]);
                                    break;
            case LAST_WALL_TYPE:    //fastImage.drawVerticalLine(x, 0, Screen.HEIGHT / 2 - height, CEIL_COLOR_RGB);
                                    //fastImage.drawVerticalLine(x, Screen.HEIGHT / 2 + height, Screen.HEIGHT - 1, FLOOR_COLOR_RGB);
                                    for (int i = 0; i <= Screen.HEIGHT / 2 - height; i++)
                                        fastImage.setPixel(x, i, ceil[c][i]);
                                    for (int i = height; i < Screen.HEIGHT / 2; i++)
                                        fastImage.setPixel(x, Screen.HEIGHT / 2 + i, floor[c][i]);
//                                     graphics.setColor(CEIL_COLOR);
//                                     graphics.drawLine(x, 0, x, Screen.HEIGHT / 2 - height);
//                                     graphics.setColor(Color.GRAY);
//                                     graphics.drawLine(x, Screen.HEIGHT / 2 + height, x, Screen.HEIGHT);
            case TREE_TYPE:
            case OBJECT_TYPE:
            case DEFAULT_TYPE:      if (otherSide instanceof ElevatorWall && ((ElevatorWall)otherSide).hasAccessToThisFloor())
                                        ((ElevatorWall)otherSide).draw(fastImage, columnInImage, c, Screen.HEIGHT / 2 - height, height, fade, (DynamicWall)wall);
                                    else
                                        wall.draw(fastImage, columnInImage, c, Screen.HEIGHT / 2 - height, height, fade);
        }
    }
    
//     private final int[] CEIL_COLOR_RGB = new int[]{ CEIL_COLOR.getRed(), CEIL_COLOR.getGreen(), CEIL_COLOR.getBlue(), 255 };
//     private final int[] FLOOR_COLOR_RGB = new int[]{ Color.GRAY.getRed(), Color.GRAY.getGreen(), Color.GRAY.getBlue(), 255 };
    
//     private static int maxH = 0;
}